@@ -238,6 +238,7 @@ virCPUDefCopyWithoutModel(const virCPUDef *cpu)
     copy->mode = cpu->mode;
     copy->match = cpu->match;
     copy->check = cpu->check;
+    copy->deprecated_feats = cpu->deprecated_feats;
     copy->fallback = cpu->fallback;
     copy->sockets = cpu->sockets;
     copy->dies = cpu->dies;
@@ -450,6 +451,11 @@ virCPUDefParseXML(xmlXPathContextPtr ctxt,
         if (virXMLPropEnum(ctxt->node, "check", virCPUCheckTypeFromString,
                            VIR_XML_PROP_NONE, &def->check) < 0)
             return -1;
+
+        if (virXMLPropTristateSwitch(ctxt->node, "deprecated_features",
+                                     VIR_XML_PROP_NONE,
+                                     &def->deprecated_feats) < 0)
+            return -1;
     }
 
     if (def->type == VIR_CPU_TYPE_HOST) {
@@ -748,6 +754,11 @@ virCPUDefFormatBufFull(virBuffer *buf,
             virBufferAsprintf(&attributeBuf, " migratable='%s'",
                               virTristateSwitchTypeToString(def->migratable));
         }
+
+        if (def->deprecated_feats) {
+            virBufferAsprintf(&attributeBuf, " deprecated_features='%s'",
+                              virTristateSwitchTypeToString(def->deprecated_feats));
+        }
     }
 
     /* Format children */
@@ -938,13 +949,13 @@ virCPUDefAddFeatureInternal(virCPUDef *def,
     return 0;
 }
 
-int
+void
 virCPUDefUpdateFeature(virCPUDef *def,
                        const char *name,
                        int policy)
 {
-    return virCPUDefAddFeatureInternal(def, name, policy,
-                                       VIR_CPU_ADD_FEATURE_MODE_UPDATE);
+    virCPUDefAddFeatureInternal(def, name, policy,
+                                VIR_CPU_ADD_FEATURE_MODE_UPDATE);
 }
 
 int
@@ -957,13 +968,13 @@ virCPUDefAddFeature(virCPUDef *def,
 }
 
 
-int
+void
 virCPUDefAddFeatureIfMissing(virCPUDef *def,
                              const char *name,
                              int policy)
 {
-    return virCPUDefAddFeatureInternal(def, name, policy,
-                                       VIR_CPU_ADD_FEATURE_MODE_NEW);
+    virCPUDefAddFeatureInternal(def, name, policy,
+                                VIR_CPU_ADD_FEATURE_MODE_NEW);
 }
 
 
